==> operation-prime-2.0.1-20140325.113557-7-sources.jar: org/gcube/data/analysis/tabulardata/operation/column/ChangeToMeasureColumn.java <==
package org.gcube.data.analysis.tabulardata.operation.column;

import java.sql.SQLException;

import org.gcube.data.analysis.tabulardata.cube.CubeManager;
import org.gcube.data.analysis.tabulardata.cube.data.connection.DatabaseConnectionProvider;
import org.gcube.data.analysis.tabulardata.cube.tablemanagers.TableCreator;
import org.gcube.data.analysis.tabulardata.model.column.Column;
import org.gcube.data.analysis.tabulardata.model.column.type.AttributeColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.IdColumnType;
import org.gcube.data.analysis.tabulardata.model.datatype.DataType;
import org.gcube.data.analysis.tabulardata.model.table.Table;
import org.gcube.data.analysis.tabulardata.operation.SQLHelper;
import org.gcube.data.analysis.tabulardata.operation.datatype.TypeTransitionSQLHandler;
import org.gcube.data.analysis.tabulardata.operation.invocation.OperationInvocation;
import org.gcube.data.analysis.tabulardata.operation.validation.ColumnTypeCastValidatorFactory;
import org.gcube.data.analysis.tabulardata.operation.worker.ImmutableWorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.Worker;
import org.gcube.data.analysis.tabulardata.operation.worker.WorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.exceptions.WorkerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ChangeToMeasureColumn extends Worker {

	private static final Logger log = LoggerFactory.getLogger(ChangeToAttributeColumn.class);

	private DatabaseConnectionProvider connectionProvider;

	private CubeManager cubeManager;

	private Table targetTable;

	private Column targetColumn;

	private DataType targetType;

	private Table newTable;


	public ChangeToMeasureColumn(OperationInvocation invocation, CubeManager cubeManager,
			DatabaseConnectionProvider connectionProvider) {
		super(invocation);
		this.cubeManager = cubeManager;
		this.connectionProvider = connectionProvider;
	}

	@Override
	protected WorkerResult execute() throws WorkerException {
		retrieveParameters();
		updateProgress(0.1f);
		createNewTable();
		updateProgress(0.5f);
		fillNewTableWithData();
		return new ImmutableWorkerResult(newTable);
	}

	private void retrieveParameters() {
		targetType = (DataType) getSourceInvocation().getParameterInstances().get(
				ColumnTypeCastValidatorFactory.TARGET_TYPE_PARAMETER.getIdentifier());
		targetTable = cubeManager.getTable(getSourceInvocation().getTargetTableId());
		targetColumn = targetTable.getColumnById(getSourceInvocation().getTargetColumnId());
	}

	@SuppressWarnings("unchecked")
	private void createNewTable() {
		TableCreator tableCreator = cubeManager.createTable(targetTable.getTableType());
		log.debug("Column to remove: " + targetColumn);
		for (Column oldColumn : targetTable.getColumnsExceptTypes(IdColumnType.class)) {
			Column newColumn = null;
			if (oldColumn.equals(targetColumn)) {
				newColumn = new Column(targetColumn.getLocalId(), targetType, new AttributeColumnType());
				newColumn.setAllMetadata(targetColumn.getAllMetadata());
				newColumn.setName(targetColumn.getName());
			} else {
				newColumn = oldColumn;
			}
			tableCreator.addColumn(newColumn);
		}
		newTable = tableCreator.create();
		log.trace("Empty table created:\n" + newTable);
	}

	private void fillNewTableWithData() throws WorkerException {
		String sqlCommand = generateSQLFillCommand();
		try {
			SQLHelper.executeSQLCommand(sqlCommand, connectionProvider);
		} catch (SQLException e) {
			log.error("Unable to execute sql query", e);
//			if (e.getErrorCode() == 0)
//				throw createFallBackWorker();
			throw new WorkerException("Unable to fill new table with data", e);
		}
	}

//	private WorkerException createFallBackWorker() throws WorkerException {
//		OperationDescriptor descriptor = fallbackValidationFactory.getOperationDescriptor();
//		OperationInvocation invocation = InvocationCreator.getCreator(descriptor).setTargetTable(targetTable.getId())
//				.setTargetColumn(targetColumn.getLocalId()).setParameters(getFallbackParameters()).create();
//		try {
//			return new WorkerException("Data seems to be not suitable for transformation",
//					fallbackValidationFactory.createWorker(invocation));
//		} catch (InvalidInvocationException e) {
//			log.error("Create invalid invocation for fallback worker. This should not happen.",e);
//			throw new WorkerException("Unable to create fallback worker",e);
//		}
//	}
//
//	private Map<String, Object> getFallbackParameters() {
//		Map<String, Object> result = new HashMap<String, Object>();
//		result.put(ColumnTypeCastValidatorFactory.TARGET_TYPE_PARAMETER.getIdentifier(), targetType);
//		return result;
//	}

	private String generateSQLFillCommand() {
		TypeTransitionSQLHandler typeTransitionHandler = TypeTransitionSQLHandler.getHandler(
				targetColumn.getDataType(), targetType);
		log.debug("Using transition handler: " + typeTransitionHandler.getClass().getSimpleName());
		return typeTransitionHandler.getCopyDataSQLCommand(newTable, targetTable, targetColumn);
	}

}

==> operation-prime-2.0.5-20140327.172558-1-sources.jar: org/gcube/data/analysis/tabulardata/operation/column/ChangeToMeasureColumn.java <==
package org.gcube.data.analysis.tabulardata.operation.column;

import java.sql.SQLException;
import java.util.Collection;

import org.gcube.data.analysis.tabulardata.cube.CubeManager;
import org.gcube.data.analysis.tabulardata.cube.data.connection.DatabaseConnectionProvider;
import org.gcube.data.analysis.tabulardata.cube.tablemanagers.TableCreator;
import org.gcube.data.analysis.tabulardata.model.column.Column;
import org.gcube.data.analysis.tabulardata.model.column.type.AttributeColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.IdColumnType;
import org.gcube.data.analysis.tabulardata.model.datatype.DataType;
import org.gcube.data.analysis.tabulardata.model.metadata.column.ColumnMetadata;
import org.gcube.data.analysis.tabulardata.model.table.Table;
import org.gcube.data.analysis.tabulardata.operation.SQLHelper;
import org.gcube.data.analysis.tabulardata.operation.datatype.TypeTransitionSQLHandler;
import org.gcube.data.analysis.tabulardata.operation.invocation.OperationInvocation;
import org.gcube.data.analysis.tabulardata.operation.validation.ColumnTypeCastValidatorFactory;
import org.gcube.data.analysis.tabulardata.operation.worker.ImmutableWorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.Worker;
import org.gcube.data.analysis.tabulardata.operation.worker.WorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.exceptions.WorkerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ChangeToMeasureColumn extends Worker {

	private static final Logger log = LoggerFactory.getLogger(ChangeToAttributeColumn.class);

	private DatabaseConnectionProvider connectionProvider;

	private CubeManager cubeManager;

	private Table targetTable;

	private Column targetColumn;

	private DataType targetType;

	private Table newTable;


	public ChangeToMeasureColumn(OperationInvocation invocation, CubeManager cubeManager,
			DatabaseConnectionProvider connectionProvider) {
		super(invocation);
		this.cubeManager = cubeManager;
		this.connectionProvider = connectionProvider;
	}

	@Override
	protected WorkerResult execute() throws WorkerException {
		retrieveParameters();
		updateProgress(0.1f);
		createNewTable();
		updateProgress(0.5f);
		fillNewTableWithData();
		return new ImmutableWorkerResult(newTable);
	}

	private void retrieveParameters() {
		targetType = (DataType) getSourceInvocation().getParameterInstances().get(
				ColumnTypeCastValidatorFactory.TARGET_TYPE_PARAMETER.getIdentifier());
		targetTable = cubeManager.getTable(getSourceInvocation().getTargetTableId());
		targetColumn = targetTable.getColumnById(getSourceInvocation().getTargetColumnId());
	}

	@SuppressWarnings("unchecked")
	private void createNewTable() {
		TableCreator tableCreator = cubeManager.createTable(targetTable.getTableType());
		log.debug("Column to remove: " + targetColumn);
		for (Column oldColumn : targetTable.getColumnsExceptTypes(IdColumnType.class)) {
			Column newColumn = null;
			if (oldColumn.equals(targetColumn)) {
				newColumn = new Column(targetColumn.getLocalId(), targetType, new AttributeColumnType());
				
				Collection<ColumnMetadata> toSetMetadata=targetColumn.getAllMetadata();
				if(getSourceInvocation().getParameterInstances().containsKey(ChangeColumnTypeTransformationFactory.ADDITIONAL_META_PARAMETER.getIdentifier())){
					Object additional=getSourceInvocation().getParameterInstances().get(ChangeColumnTypeTransformationFactory.ADDITIONAL_META_PARAMETER.getIdentifier());
					if(additional instanceof Collection<?>) toSetMetadata.addAll((Collection<? extends ColumnMetadata>) additional);
					else toSetMetadata.add((ColumnMetadata) additional);
				}				
				newColumn.setAllMetadata(toSetMetadata);
				
				
				newColumn.setName(targetColumn.getName());
			} else {
				newColumn = oldColumn;
			}
			tableCreator.addColumn(newColumn);
		}
		newTable = tableCreator.create();
		log.trace("Empty table created:\n" + newTable);
	}

	private void fillNewTableWithData() throws WorkerException {
		String sqlCommand = generateSQLFillCommand();
		try {
			SQLHelper.executeSQLCommand(sqlCommand, connectionProvider);
		} catch (SQLException e) {
			log.error("Unable to execute sql query", e);
//			if (e.getErrorCode() == 0)
//				throw createFallBackWorker();
			throw new WorkerException("Unable to fill new table with data", e);
		}
	}

//	private WorkerException createFallBackWorker() throws WorkerException {
//		OperationDescriptor descriptor = fallbackValidationFactory.getOperationDescriptor();
//		OperationInvocation invocation = InvocationCreator.getCreator(descriptor).setTargetTable(targetTable.getId())
//				.setTargetColumn(targetColumn.getLocalId()).setParameters(getFallbackParameters()).create();
//		try {
//			return new WorkerException("Data seems to be not suitable for transformation",
//					fallbackValidationFactory.createWorker(invocation));
//		} catch (InvalidInvocationException e) {
//			log.error("Create invalid invocation for fallback worker. This should not happen.",e);
//			throw new WorkerException("Unable to create fallback worker",e);
//		}
//	}
//
//	private Map<String, Object> getFallbackParameters() {
//		Map<String, Object> result = new HashMap<String, Object>();
//		result.put(ColumnTypeCastValidatorFactory.TARGET_TYPE_PARAMETER.getIdentifier(), targetType);
//		return result;
//	}

	private String generateSQLFillCommand() {
		TypeTransitionSQLHandler typeTransitionHandler = TypeTransitionSQLHandler.getHandler(
				targetColumn.getDataType(), targetType);
		log.debug("Using transition handler: " + typeTransitionHandler.getClass().getSimpleName());
		return typeTransitionHandler.getCopyDataSQLCommand(newTable, targetTable, targetColumn);
	}

}
